package Bio::IO::CBust;
use IO::File;
use Bio::IO::CBust::Record;
use Bio::IO::CBust::Matrix;
use Bio::IO::CBust::Cluster;


sub new {
  my $class = shift;
  my %param = @_;
  unless($param{file}) {
    print STDERR "file option must be specified!\n";
    return;
  }
  my $self = \%param;
  bless $self, $class;
  return $self;
}

sub next_record {
  my $self = shift;
  my $fh = $self->fh;
  if($self->{previous_line} =~ /^\s*>/) {
    my $l = $self->{previous_line};
    $self->{previous_line} = "";
    return $self->_record($l);
  } else {
    while(<$fh>) {
      if(/^\s*>/) {
	return $self->_record($_);
      }
    }
  }
  return;
}
our @MR = qw(name slocation elocation strand score sequence);
sub _record {
  my $self = shift;
  my $first_line = shift;
  my $r = {};
  my($gene, $length) = ($first_line =~ />(.*?) \((.*)\)/);
  return unless($gene);
  $self->{record} = new Bio::IO::CBust::Record(gene => $gene, len => $length);
  my $fh = $self->fh;
  my $cluster;
  my $start = 0;
  my $cobj;
  while(<$fh>) {
    s/^\s*|\s*$//g;
    if(/^\s*$/) {
      $cluster = "";
      $start = 0;
      next;
    } elsif(/^\s*>/) {
        $self->{previous_line} = $_;
	return $self->{record};   
    }
    unless($cluster) {
      if(/^CLUSTER/) {
	$cluster = $_;
	$cobj = new Bio::IO::CBust::Cluster(name => $cluster);      
	$self->{record}->add_cluster($cobj);
      }
    }
    if(/^Location\:\s*(\d+)\s*to\s*(\d+)/i) {
      $cobj->location( [$1, $2] );
    } elsif(/^Score\:\s*(\S+)/) {
      $cobj->score($1);
      $start = 1;
    } elsif($start) {
      my @f = split /\s+/, $_;
      if(@f == 1) {
        $cobj->sequence($_);
      } elsif(@f == 2) {
        
      } elsif(@f > 2) {
        my $i = $#MR;
        $cobj->add_matrix(new Bio::IO::CBust::Matrix(map { $MR[$_] => $f[$_] } (0..$i))); 
      }
    }
  }
  
  return $self->{record};
}
sub _cluster {
  my $self = shift;
  my $cluster = shift;
  my $fh = $self->fh;
  
}
sub fh {
  my $self = shift;
  unless($self->{fh}) {
    my $fh = new IO::File "<" . $self->{file}  or die "ERROR[" . $self->{file} . "]: $!";
    $self->{fh} = $fh;
  }
  return $self->{fh};
}

1;
